# Personal Temperature Controller and Logger
# A Debby Solutions program
# www.debbysolutions.com
# 1/3/2022
#
from guizero import *                       #for GUI creation
import RPi.GPIO as GPIO                     #for GPIO control
import Adafruit_DHT as dht                  #for DHT-22 control                
from datetime import *                      #will use for getting current date and time
from time import sleep                      #will use sleep to set delay

#Suppress the GPIO warnings
GPIO.setwarnings(False)

#Set GPIO23 as an output to control the AC Power Relay Module
GPIO.setmode(GPIO.BCM)                      #Use GPIO pin numbering
GPIO.setup(23,GPIO.OUT)                     #set GPIO23 as output
#Set DHT variable to indicate that GPIO17 will be used for DHT-22 sensor
DHT = 17

#set up program variables
count = 0
heaterState = "OFF"
desiredTemperature = 70                     #initial temperature set point in degrees F
desiredVariance = 1.0                       #heater will be controlled within 1/2 of this temperature range
currentDate = date.today()                  #the current date will be used as log file name

#Create file and write out header row in new CSV file
filename = str(currentDate) + ".csv"
csv = open(filename, 'w')
csv.write("Timestamp, Temperature, Humidity, Heater State\n")
csv.close()

#function to read data from the DHT-22 sensor to turn heater ON/OFF and also write value out to log file
def read_temp():
    global heaterState, filename, currentDate, count
    today = date.today()
    #start a new file if we are on a different date
    if (today != currentDate):                                  #start a new file if we are on a different date
        filename = str(today) + ".csv"
        currentDate = today
        csv = open(filename, 'w')
        csv.write("Timestamp, Temperature, Humidity, Heater State\n")
        csv.close()
    #Read Temp and Hum from DHT22
    h,t = dht.read_retry(dht.DHT22, DHT)                        #read humidity and temperature from DHT-22 sensor
    ft = round((t*9/5)+32,1)                                    #convert temperature from C to F
    if (ft < desiredTemperature - desiredVariance/2):           #if temperature falls below desired temperature
        GPIO.output(23,GPIO.HIGH)                                               #turn heater ON
        heaterState = "ON"
        #print('Temp={0:0.1f}*C  or {1:0.1f}*F Humidity={2:0.1f}% Heater ON'.format(t,ft,h))
    if (ft > desiredTemperature + desiredVariance/2):           #if temperature rises above desired temperature
        GPIO.output(23,GPIO.LOW)                                                #turn heater OFF
        heaterState = "OFF"
        #print('Temp={0:0.1f}*C  or {1:0.1f}*F Humidity={2:0.1f}% Heater OFF'.format(t,ft,h))
    if (count >= 30):                                            #if number of times we have read from DHT-22 is greater than 30
        temp_c = str(round(ft,1))                                               #convert temperature value to string
        hum_c = str(round(h,1))                                                 #convert humidity value to string
        now = datetime.now()                                                    #get current time, and
        entry = now.strftime("%H:%M")                                           #convert to hour and minute format
        entry = entry + "," + temp_c + "," + hum_c + "," + heaterState + "\n"   #create complete string, and
        #append log entry to file
        csv = open(filename, 'a')                                               #append to log file
        try:
            csv.write(entry)
        finally:
            csv.close()
        count = 0
    else:                                                       #if less than 30 times,
        count = count + 1                                                       #increment count
    return ft                                                   #return the acquired temperature in F

#function to update the desired and current temperature values on the GUI
def update_label():
    currentTemp.value = read_temp()                             #call the read_temp() function to acquire the current temperature in F
    desiredTemp.value = desiredTemperature                      #update desired temperature since that could have been changed
    # recursive call
    currentTemp.after(1000, update_label)                       #will call itself after 1000 mS recursively to keep updating GUI

#function to increase desired temperature by one degree
def upTemp():
    global desiredTemperature
    desiredTemperature = desiredTemperature + 1

#function to decrease desired temperature by one degree
def downTemp():
    global desiredTemperature
    desiredTemperature = desiredTemperature - 1

#Main execution code to set up the GUI window and displaying it
if __name__=='__main__':
    #instantiate a GUI window with title, size and format
    app = App(title="dTemp - a Debby Solution",
              height=180,
              width=450,
              layout='grid')
    #define the GUI window content
    DesiredTitle = Text(app, 'Desired Temperature', grid=[0,0])                             #set up title of Desired Temperature and location, and
    desiredTemp = Text(app, "xx", size=80 , color="gray", grid=[0,1])                       #set "xx" as placeholder with desired size and location
    upTempButton = PushButton(app, command=upTemp, text=' '+chr(94)+' ', grid=[1,0])        #set up ^ button and will call upTemp function when touched
    downTempButton = PushButton(app, command=downTemp, text=' '+chr(118)+' ', grid=[1,1])   #set up v button and will call downTemp function when touched
    Space1 = Text(app, '   ', grid=[2,0])                                                   #add some space between buttons and Current Temperature
    Space2 = Text(app, '   ', grid=[2,1])                                                   #add some space between buttons and Current Temperature
    CurrentTitle = Text(app, 'Current Temperature', grid=[3,0])                             #set up title of Current Temperature and location, and
    currentTemp = Text(app, "xx", size=80, grid=[3,1])                                      #set "xx" as placeholder with desired size location
    currentTemp.after(1000, update_label)
    #display the GUI window
    app.display()
